#include "thsq.h"
#include "abc123.h"
#include "ti-lib.h"
#include "gpio-interrupt.h"
/*---------------------------------------------------------------------------*/
PROCESS(abc123_process, "ABC123 Sensor Process");

abc123_callback_t callback = NULL;
static int sda = IOID_UNUSED;
static int scl = IOID_UNUSED;
static int irq = IOID_UNUSED;
static uint16_t last_sample = 0;
static int inited = 0;

/* tell sensor to start sampling */
static void start_sample(void);

/* read last sample from sensor */
static uint16_t read_sample(void);

/* GPIO interrupt handler */
static void irq_handler(uint8_t ioid);

/* configuration for the button interrupt; rising edge when done */
#define IRQ_GPIO_CFG            (IOC_CURRENT_2MA  | IOC_STRENGTH_AUTO | \
                                 IOC_IOPULL_UP    | IOC_SLEW_DISABLE  | \
                                 IOC_HYST_DISABLE | IOC_RISING_EDGE   | \
                                 IOC_INT_ENABLE   | IOC_IOMODE_NORMAL | \
                                 IOC_INPUT_ENABLE)
/*---------------------------------------------------------------------------*/
int
abc123_init(int i2c_sda, int i2c_scl, int irqpin)
{
  if(inited) {
    return -1;
  }

  sda = i2c_sda;
  scl = i2c_scl;
  irq = irqpin;

  /* set up i2c */
  // left out for brevity

  /* init GPIO for irq */
  ti_lib_gpio_clear_event_dio(irq);
  ti_lib_rom_ioc_pin_type_gpio_input(irq);
  ti_lib_rom_ioc_port_configure_set(irq, IOC_PORT_GPIO, IRQ_GPIO_CFG);
  gpio_interrupt_register_handler(irq, irq_handler);
  ti_lib_rom_ioc_int_enable(irq);

  inited = 1;

  return 0;
}
/*---------------------------------------------------------------------------*/
int
abc123_start_sample(abc123_callback_t cb)
{
  if(inited == 0) {
    return -1;
  }

  process_start(&abc123_process, NULL);
  return 0;
}
/*---------------------------------------------------------------------------*/
uint16_t
abc123_get_last(void)
{
  if(inited == 0) {
    return 0;
  }

  return last_sample;
}
/*---------------------------------------------------------------------------*/
PROCESS_THREAD(abc123_process, ev, data)
{
  PROCESS_BEGIN();

  /* tell sensor to sample, then wait for it to finish */
  start_sample();
  PROCESS_WAIT_EVENT_UNTIL(ev == PROCESS_EVENT_POLL);

  /* read out and invoke callback */
  last_sample = read_sample();
  if(callback != NULL) {
    callback(last_sample);
  }

  PROCESS_END();
}
/*---------------------------------------------------------------------------*/
static uint16_t
read_sample(void)
{
  uint16_t ret;
  /* ... here we read the sensor sample over i2c */
  ret = 0;/* XXX left out for brevity */
  return ret;
}
/*---------------------------------------------------------------------------*/
static void
start_sample(void)
{
  /* ... here we command the sensor over i2c to start the sample process */
  /* XXX left out for brevity */
}
/*---------------------------------------------------------------------------*/
static void
irq_handler(uint8_t ioid)
{
  /*
   * since the irq callback can be used for more than one GPIO, we check this
   * here, although this is really not necessary since we only use it for one.
   */
  if(ioid == irq) {
    /*
     * In the interrupt context, we do the very least possible. For example,
     * we do not perform any i2c (or similar) communication. We could timestamp
     * this here if we wanted though.
     */
    process_poll(&abc123_process);
  }
}
/*---------------------------------------------------------------------------*/
