#ifndef ABC123_H
#define ABC123_H
/*---------------------------------------------------------------------------*/
#include "thsq.h"
#include "ti-lib.h"
/*---------------------------------------------------------------------------*/
/*
 * Driver for the ABC123 sensor.
 * Longer description here...
 */
/*---------------------------------------------------------------------------*/
/* callback definition; will be invoked when the sensor is done sampling */
typedef void (*abc123_callback_t)(uint16_t);

/*
 * Init the sensor. Configures the sensor and leaves it in deep sleep.
 * Can only be called once. Must be called before any other function is called.
 * arguments
 *    i2c_sda      The GPIO used for I2C SDA
 *    i2c_scl      The GPIO used for I2C SCL
 *    irqpin       The GPIO used for interrupt, trigger on rising edge
 * returns
 *    -1    failed, already initialized
 *    0     success
 */
int abc123_init(int i2c_sda, int i2c_scl, int irqpin);

/*
 * Start a sensor sample. The ABC123 sensor will signal an interrupt when done.
 * The sensor sample is automatically read out and buffered, available through
 * calling abc123_get_last()
 * arguments
 *    cb      The callback that will be invoked when sample is read out
 *            the callback will be provided the sample value as argument
 * returns
 *    -1    failed, not initialized
 *    0     success
 */
int abc123_start_sample(abc123_callback_t cb);

/*
 * Get the last sample value.
 * returns
 *    0 until first sample is taken
 *    then, the last sensor sample
 */
uint16_t abc123_get_last(void);
/*---------------------------------------------------------------------------*/
#endif  /* ABC123_H */
