#include "thsq.h"
#include "abc123.h"
#include "power-control.h"
/*---------------------------------------------------------------------------*/
static void
sensor_callback(uint16_t val)
{
  /*
   * We are invoked when the abc123 sensor has sampled. This is not in an
   * interrupt context, so it's ok to set the variable and push the data.
   * For the variable name, we use a semantically better name, not "abc123".
   * Let's use "bmp" as in "barometric pressure".
   */
  thsq_sset("bmp", (int)val);
  thsq_push();
}
/*---------------------------------------------------------------------------*/
static void
callback_thsq(enum thsq_reason r, const char *str, int len)
{
  if(r == THSQ_PERIOD) {
    /*
     * Time to collect sensor data and push to the backend. We start our sensor
     * and push when done.
     */
    if(abc123_start_sample(sensor_callback) < 0) {
      /* handle error */
    }

  } else if(r == THSQ_KEYVAL) {
    if(strncmp(str, "psu", 3) == 0 && thsq_exists("psu")) {
      /* PSU variable updated; control PSU accordingly */
      if(thsq_get("psu") > 0) {
        power_control_on();
      } else {
        power_control_off();
      }
    }
  }
}
/*---------------------------------------------------------------------------*/
void
app(void)
{
  thsq_set_default_mode("feather");

  /* init the ABC123 sensor driver */
  if(abc123_init(IOID_28, IOID_29, IOID_30) < 0) {
    /* handle error */
  }

  /* init the PSU power control driver */
  power_control_init(IOID_23);
  power_control_on();

  /* Set up the callback */
  static struct thsq_callback cb;
  thsq_add_callback(&cb, callback_thsq);
}
/*---------------------------------------------------------------------------*/
